@@ -1,4 +1,5 @@
 import base64
+import contextlib
 import io
 import json
 import os.path as osp
@@ -15,13 +16,25 @@ from labelme import utils
 PIL.Image.MAX_IMAGE_PIXELS = None
 
 
+@contextlib.contextmanager
+def open(name, mode):
+    assert mode in ["r", "w"]
+    if PY2:
+        mode += "b"
+        encoding = None
+    else:
+        encoding = "utf-8"
+    yield io.open(name, mode, encoding=encoding)
+    return
+
+
 class LabelFileError(Exception):
     pass
 
 
 class LabelFile(object):
 
-    suffix = '.json'
+    suffix = ".json"
 
     def __init__(self, filename=None):
         self.shapes = []
@@ -36,7 +49,7 @@ class LabelFile(object):
         try:
             image_pil = PIL.Image.open(filename)
         except IOError:
-            logger.error('Failed opening image file: {}'.format(filename))
+            logger.error("Failed opening image file: {}".format(filename))
             return
 
         # apply orientation to image according to exif
@@ -45,77 +58,78 @@ class LabelFile(object):
         with io.BytesIO() as f:
             ext = osp.splitext(filename)[1].lower()
             if PY2 and QT4:
-                format = 'PNG'
-            elif ext in ['.jpg', '.jpeg']:
-                format = 'JPEG'
+                format = "PNG"
+            elif ext in [".jpg", ".jpeg"]:
+                format = "JPEG"
             else:
-                format = 'PNG'
+                format = "PNG"
             image_pil.save(f, format=format)
             f.seek(0)
             return f.read()
 
     def load(self, filename):
         keys = [
-            'version',
-            'imageData',
-            'imagePath',
-            'shapes',  # polygonal annotations
-            'flags',   # image level flags
-            'imageHeight',
-            'imageWidth',
+            "version",
+            "imageData",
+            "imagePath",
+            "shapes",  # polygonal annotations
+            "flags",  # image level flags
+            "imageHeight",
+            "imageWidth",
         ]
         shape_keys = [
-            'label',
-            'points',
-            'group_id',
-            'shape_type',
-            'flags',
+            "label",
+            "points",
+            "group_id",
+            "shape_type",
+            "flags",
         ]
         try:
-            with open(filename, 'rb' if PY2 else 'r') as f:
+            with open(filename, "r") as f:
                 data = json.load(f)
-            version = data.get('version')
+            version = data.get("version")
             if version is None:
                 logger.warn(
-                    'Loading JSON file ({}) of unknown version'
-                    .format(filename)
+                    "Loading JSON file ({}) of unknown version".format(
+                        filename
+                    )
                 )
-            elif version.split('.')[0] != __version__.split('.')[0]:
+            elif version.split(".")[0] != __version__.split(".")[0]:
                 logger.warn(
-                    'This JSON file ({}) may be incompatible with '
-                    'current labelme. version in file: {}, '
-                    'current version: {}'.format(
+                    "This JSON file ({}) may be incompatible with "
+                    "current labelme. version in file: {}, "
+                    "current version: {}".format(
                         filename, version, __version__
                     )
                 )
 
-            if data['imageData'] is not None:
-                imageData = base64.b64decode(data['imageData'])
+            if data["imageData"] is not None:
+                imageData = base64.b64decode(data["imageData"])
                 if PY2 and QT4:
                     imageData = utils.img_data_to_png_data(imageData)
             else:
                 # relative path from label file to relative path from cwd
-                imagePath = osp.join(osp.dirname(filename), data['imagePath'])
+                imagePath = osp.join(osp.dirname(filename), data["imagePath"])
                 imageData = self.load_image_file(imagePath)
-            flags = data.get('flags') or {}
-            imagePath = data['imagePath']
+            flags = data.get("flags") or {}
+            imagePath = data["imagePath"]
             self._check_image_height_and_width(
-                base64.b64encode(imageData).decode('utf-8'),
-                data.get('imageHeight'),
-                data.get('imageWidth'),
+                base64.b64encode(imageData).decode("utf-8"),
+                data.get("imageHeight"),
+                data.get("imageWidth"),
             )
             shapes = [
                 dict(
-                    label=s['label'],
-                    points=s['points'],
-                    shape_type=s.get('shape_type', 'polygon'),
-                    flags=s.get('flags', {}),
-                    group_id=s.get('group_id'),
+                    label=s["label"],
+                    points=s["points"],
+                    shape_type=s.get("shape_type", "polygon"),
+                    flags=s.get("flags", {}),
+                    group_id=s.get("group_id"),
                     other_data={
                         k: v for k, v in s.items() if k not in shape_keys
-                    }
+                    },
                 )
-                for s in data['shapes']
+                for s in data["shapes"]
             ]
         except Exception as e:
             raise LabelFileError(e)
@@ -138,14 +152,14 @@ class LabelFile(object):
         img_arr = utils.img_b64_to_arr(imageData)
         if imageHeight is not None and img_arr.shape[0] != imageHeight:
             logger.error(
-                'imageHeight does not match with imageData or imagePath, '
-                'so getting imageHeight from actual image.'
+                "imageHeight does not match with imageData or imagePath, "
+                "so getting imageHeight from actual image."
             )
             imageHeight = img_arr.shape[0]
         if imageWidth is not None and img_arr.shape[1] != imageWidth:
             logger.error(
-                'imageWidth does not match with imageData or imagePath, '
-                'so getting imageWidth from actual image.'
+                "imageWidth does not match with imageData or imagePath, "
+                "so getting imageWidth from actual image."
             )
             imageWidth = img_arr.shape[1]
         return imageHeight, imageWidth
@@ -162,7 +176,7 @@ class LabelFile(object):
         flags=None,
     ):
         if imageData is not None:
-            imageData = base64.b64encode(imageData).decode('utf-8')
+            imageData = base64.b64encode(imageData).decode("utf-8")
             imageHeight, imageWidth = self._check_image_height_and_width(
                 imageData, imageHeight, imageWidth
             )
@@ -183,7 +197,7 @@ class LabelFile(object):
             assert key not in data
             data[key] = value
         try:
-            with open(filename, 'wb' if PY2 else 'w') as f:
+            with open(filename, "w") as f:
                 json.dump(data, f, ensure_ascii=False, indent=2)
             self.filename = filename
         except Exception as e:
